import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Chip, ListItemDecorator, Menu, MenuItem } from "@mui/joy";
import { AppBar, Box, Divider, Toolbar, Typography } from "@mui/material";
import { useEffect, useRef, useState, } from "react";
import { useLocation, useNavigate } from "react-router-dom";
export default function NerdNavigationBar({ items, navigationBarIndex, chipVariant, }) {
    const navigate = useNavigate();
    const location = useLocation();
    const menus = useRef([]);
    const [menuIndex, setMenuIndex] = useState(null);
    const anchorRef = useRef([]);
    const childrenMenus = useRef({});
    for (let i = 0; i < items.length; i++) {
        if (!childrenMenus.current[i])
            childrenMenus.current[i] = [];
    }
    const [childrenMenuIndex, setChildrenMenuIndex] = useState(null);
    const handleMenuClick = (index) => {
        setMenuIndex(index);
    };
    const handleChildrenMenuClick = (index) => {
        setChildrenMenuIndex(index);
    };
    const handleNavigationClick = (to) => {
        navigate(to);
    };
    const handleMenuClose = () => {
        if (typeof menuIndex === "number" &&
            typeof childrenMenuIndex === "number" &&
            items[menuIndex] &&
            items[menuIndex].children) {
            const tempChildren = items[menuIndex].children;
            if (!tempChildren ||
                (tempChildren && !tempChildren[childrenMenuIndex].children)) {
                setMenuIndex(null);
            }
        }
        else {
            setMenuIndex(null);
        }
    };
    const handleChildrenMenuClose = () => {
        setMenuIndex(null);
        setChildrenMenuIndex(null);
    };
    useEffect(() => {
        if (childrenMenuIndex === null &&
            typeof menuIndex === "number" &&
            anchorRef.current[menuIndex]) {
            anchorRef.current[menuIndex].focus();
        }
    }, [childrenMenuIndex]);
    function isSelected(item) {
        if (item.selected) {
            return true;
        }
        else if (item.children) {
            let result = false;
            item.children.every((child) => {
                result = isSelected(child);
                if (result) {
                    return false;
                }
                else {
                    return true;
                }
            });
            return result;
        }
        else
            return false;
    }
    function isCurrentPath(item, pathname) {
        if (item.to && "/" + item.to === pathname) {
            return true;
        }
        else if (item.children) {
            let result = false;
            item.children.every((child) => {
                result = isCurrentPath(child, pathname);
                if (result) {
                    return false;
                }
                else {
                    return true;
                }
            });
            return result;
        }
        else {
            return false;
        }
    }
    function navigationChip({ item, index, }) {
        return (_jsx(Chip, { variant: chipVariant ? chipVariant : "soft", color: isCurrentPath(item, location.pathname) || isSelected(item)
                ? "primary"
                : "neutral", size: "sm", startDecorator: item.startDecorator ? item.startDecorator : null, endDecorator: !item.to && !item.action && item.children ? (_jsx("i", { className: "fa-sharp fa-solid fa-caret-down" })) : null, onMouseEnter: () => {
                if (typeof menuIndex === "number") {
                    setMenuIndex(index);
                }
                setChildrenMenuIndex(null);
            }, onClick: () => {
                if (item.to) {
                    handleNavigationClick(item.to);
                }
                else if (item.action) {
                    item.action();
                }
                else if (item.children) {
                    handleMenuClick(index);
                }
            }, ref: (instance) => {
                if (instance)
                    menus.current[index] = instance;
            }, children: item.title }, index));
    }
    function menuItemBox({ item, siblings, }) {
        let startDeco = false;
        if (siblings) {
            siblings.every((sibling) => {
                if (sibling.startDecorator) {
                    startDeco = true;
                    return false;
                }
                else {
                    return true;
                }
            });
        }
        return (_jsxs(Box, { sx: {
                display: "flex",
                alignItems: "center",
                width: "100%",
            }, children: [startDeco ? (_jsx(ListItemDecorator, { children: item.startDecorator ? item.startDecorator : null })) : null, _jsxs(Box, { sx: {
                        display: "flex",
                        alignItems: "center",
                        justifyContent: "space-between",
                        width: "100%",
                    }, children: [_jsx(Typography, { color: isCurrentPath(item, location.pathname) || isSelected(item)
                                ? "primary"
                                : "neutral", sx: {
                                paddingRight: "10px",
                                paddingLeft: startDeco ? "0px" : "10px",
                            }, children: item.title }), !item.to && !item.action && item.children ? (_jsx(Box, { children: _jsx("i", { className: "fa-solid fa-angle-right" }) })) : null] })] }));
    }
    return (_jsxs(AppBar, { position: "fixed", sx: {
            background: (theme) => theme.palette.background.default,
            zIndex: navigationBarIndex ? navigationBarIndex : 1100,
            marginTop: "48px",
            boxShadow: "none",
        }, color: "transparent", children: [_jsx(Divider, {}), _jsx(Toolbar, { variant: "dense", sx: { gap: 1, overflowY: "auto" }, children: items.map((item, index) => {
                    return (_jsxs(_Fragment, { children: [navigationChip({ item: item, index: index }), !item.to && !item.action && item.children ? (_jsx(Menu, { id: `menu ${index}`, anchorEl: menus.current[index], open: menuIndex === index, onClose: handleMenuClose, placement: "bottom-start", ref: (instance) => {
                                    if (instance)
                                        anchorRef.current[index] = instance;
                                }, sx: {
                                    borderRadius: 3,
                                    zIndex: navigationBarIndex ? navigationBarIndex + 1 : 1101,
                                    background: (theme) => theme.palette.background.body,
                                }, children: item.children.map((elem, elemIndex) => {
                                    return (_jsxs(_Fragment, { children: [_jsx(MenuItem, { onClick: () => {
                                                    if (elem.to) {
                                                        handleMenuClose();
                                                        handleNavigationClick(elem.to);
                                                    }
                                                    else if (elem.action) {
                                                        handleMenuClose();
                                                        elem.action();
                                                    }
                                                    else {
                                                        handleChildrenMenuClick(elemIndex);
                                                    }
                                                }, onMouseEnter: () => {
                                                    if (!elem.children) {
                                                        setChildrenMenuIndex(null);
                                                    }
                                                    else {
                                                        setChildrenMenuIndex(elemIndex);
                                                    }
                                                }, ref: (instance) => {
                                                    if (instance) {
                                                        if (childrenMenus.current[index]) {
                                                            childrenMenus.current[index][elemIndex] =
                                                                instance;
                                                        }
                                                        else {
                                                            childrenMenus.current[index] = [];
                                                            childrenMenus.current[index][elemIndex] =
                                                                instance;
                                                        }
                                                    }
                                                }, children: menuItemBox({ item: elem, siblings: item.children }) }, elemIndex), !elem.to && !elem.action && elem.children ? (_jsx(Menu, { id: `menu child ${elemIndex}`, anchorEl: childrenMenus.current[index][elemIndex], open: menuIndex === index &&
                                                    childrenMenuIndex === elemIndex, onClose: () => {
                                                    handleChildrenMenuClose();
                                                }, placement: "right-start", sx: {
                                                    borderRadius: 3,
                                                    zIndex: navigationBarIndex
                                                        ? navigationBarIndex + 2
                                                        : 1102,
                                                    background: (theme) => theme.palette.background.body,
                                                }, children: elem.children.map((elem2, elem2Index) => {
                                                    return (_jsx(MenuItem, { onClick: () => {
                                                            if (elem2.to) {
                                                                handleChildrenMenuClose();
                                                                handleNavigationClick(elem2.to);
                                                            }
                                                            else if (elem2.action) {
                                                                handleChildrenMenuClose();
                                                                elem2.action();
                                                            }
                                                        }, children: menuItemBox({
                                                            item: elem2,
                                                            siblings: elem.children,
                                                        }) }, elem2Index));
                                                }) })) : (_jsx(_Fragment, {}))] }));
                                }) })) : (_jsx(_Fragment, {}))] }));
                }) })] }));
}
